package cn.estsh.i3plus.ext.mes.apiservice.controller.busi.bu;

import cn.estsh.i3plus.ext.mes.api.busi.bu.IMesProcessLabelMarkingService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesProcessLabelMarking;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.impp.framework.base.controller.MesBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping(CommonConstWords.BASE_URL_MES+"/process-label-marking")
@Api(tags = "工序标签标示")
public class MesProcessLabelMarkingController extends MesBaseController {

    @Autowired
    private IMesProcessLabelMarkingService processLabelMarkingService;

    @GetMapping(value = "/query-list-page")
    @ApiOperation(value = "工序标签标示分页查询")
    public ResultBean getProcessLabelMarkingPage(MesProcessLabelMarking processLabelMarking, Pager pager) {
        String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
        try {
            //请求参数非空
            if (null == processLabelMarking || null == pager) {
                return ResultBean.fail("请求参数不完整");
            }

            ResultBean resultBean = new ResultBean(true);
            return ResultBean.success("工序标签标示分页查询")
                    .setListPager(processLabelMarkingService.getProcessLabelMarkingPage(processLabelMarking, pager, organizeCode))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException imppException) {

            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增工序标签标示")
    public ResultBean insertProcessLabelMarking(MesProcessLabelMarking processLabelMarking) {
        String userName = AuthUtil.getSessionUser().getUserName();
        SessionUser sessionUser = AuthUtil.getSessionUser();
        String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
        try {
            // 属性验证
            /*ValidatorBean.beginValid(mesWorkOrder)
                    .notNull("id", mesWorkOrder.getId());*/
            processLabelMarkingService.insertProcessLabelMarking(processLabelMarking, organizeCode, userName);
            return new ResultBean(true).setMsg("新增成功");
        } catch (ImppBusiException imppException) {

            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/update")
    @ApiOperation(value = "更新工序标签标示")
    public ResultBean updateprocessLabelMarking(MesProcessLabelMarking processLabelMarking) {
        String userName = AuthUtil.getSessionUser().getUserName();
        String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
        try {
            // 属性验证
            ValidatorBean.beginValid(processLabelMarking)
                    .notNull("id", processLabelMarking.getId());
            processLabelMarkingService.updateProcessLabelMarking(processLabelMarking, organizeCode, userName);
            return new ResultBean(true).setMsg("更新成功");
        } catch (ImppBusiException imppException) {

            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
