package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesProcessCellService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesProcessCell;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesProcessCellRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.sqlpack.MesHqlPack;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;

/**
 * @Author: xiangming.liao
 * @CreateDate: 2021/01/19 15:15 下午
 * @Description:
 **/
@Service
public class MesProcessCellService implements IMesProcessCellService {

    @Autowired
    private MesProcessCellRepository mesProcessCellRDao;

    @Override
    public MesProcessCell insertMesProcessCell(MesProcessCell mesProcessCell) {
        //唯一性校验
        MesProcessCell mesProcessCellDb = mesProcessCellRDao.getByProperty(
                new String[]{MesExtConstWords.WORK_CELL_CODE, MesExtConstWords.WORK_CENTER_CODE, MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{mesProcessCell.getWorkCellCode(),
                        mesProcessCell.getWorkCenterCode(), AuthUtil.getOrganize().getOrganizeCode(), CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});

        if (null != mesProcessCellDb) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("工位代码为【%s】，生产线代码为【%s】的记录已存在"
                            , mesProcessCell.getWorkCellCode(), mesProcessCell.getWorkCenterCode())
                    .build();
        }
        ConvertBean.saveOrUpdate(mesProcessCell, AuthUtil.getSessionUser().getUserName());
        return mesProcessCellRDao.insert(mesProcessCell);
    }

    @Override
    public ListPager<MesProcessCell> queryMesProcessCellByPager(MesProcessCell mesProcessCell, Pager pager) {
        DdlPackBean packBean = MesHqlPack.getAllBaseData(AuthUtil.getOrganize().getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(mesProcessCell.getIsValid(),MesExtConstWords.IS_VALID,packBean);
        if (!StringUtils.isEmpty(mesProcessCell.getProcessCode())) {
            DdlPreparedPack.getStringLikerPack(mesProcessCell.getProcessCode(), MesExtConstWords.PROCESS_CODE, packBean);
        }
        if (!StringUtils.isEmpty(mesProcessCell.getProcessName())) {
            DdlPreparedPack.getStringLikerPack(mesProcessCell.getProcessName(), MesExtConstWords.PROCESS_NAME, packBean);
        }
        if (!StringUtils.isEmpty(mesProcessCell.getWorkCellCode())) {
            DdlPreparedPack.getStringEqualPack(mesProcessCell.getWorkCellCode(), MesExtConstWords.WORK_CELL_CODE, packBean);
        }
        if (!StringUtils.isEmpty(mesProcessCell.getWorkCenterCode())) {
            DdlPreparedPack.getStringEqualPack(mesProcessCell.getWorkCenterCode(), MesExtConstWords.WORK_CENTER_CODE, packBean);
        }
        pager = PagerHelper.getPager(pager, mesProcessCellRDao.findByHqlWhereCount(packBean));
        packBean.setOrderByStr(mesProcessCell.orderBy());
        List<MesProcessCell> mesProcessCells = mesProcessCellRDao.findByHqlWherePage(packBean, pager);
        //mesProcessCells = this.transferCodeToName(mesProcessCells);
        return new ListPager(mesProcessCells, pager);
    }

    @Override
    public MesProcessCell updateMesProcessCell(MesProcessCell mesProcessCell) {
        //对id做有效性校验
        MesProcessCell mesProcessCellDb = mesProcessCellRDao.getById(mesProcessCell.getId());
        if (mesProcessCellDb == null) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("工序代码为【%s】，工位代码为【%s】，生产线代码为【%s】的记录不存在"
                            , mesProcessCell.getProcessCode(), mesProcessCell.getWorkCellCode(), mesProcessCell.getWorkCenterCode())
                    .build();
        }

        //唯一性校验
        DdlPackBean packBean = MesHqlPack.getAllBaseData(mesProcessCell.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProcessCell.getWorkCellCode(), MesExtConstWords.WORK_CELL_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(mesProcessCell.getWorkCenterCode(), MesExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getNumNOEqualPack(mesProcessCell.getId(), MesExtConstWords.ID, packBean);
        List<MesProcessCell> MesProcessCellDbs = mesProcessCellRDao.findByHqlWhere(packBean);
        if (!CollectionUtils.isEmpty(MesProcessCellDbs)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("工位代码为【%s】，生产线代码为【%s】的记录已存在"
                            , mesProcessCell.getWorkCellCode(), mesProcessCell.getWorkCenterCode())
                    .build();
        }
        mesProcessCellDb.setProcessCode(mesProcessCell.getProcessCode());
        mesProcessCellDb.setWorkCellCode(mesProcessCell.getWorkCellCode());
        mesProcessCellDb.setWorkCenterCode(mesProcessCell.getWorkCenterCode());
        mesProcessCellDb.setWorkCenterName(mesProcessCell.getWorkCenterName());
        mesProcessCellDb.setWorkCellName(mesProcessCell.getWorkCellName());
        mesProcessCellDb.setProcessName(mesProcessCell.getProcessName());
        ConvertBean.serviceModelUpdate(mesProcessCellDb, AuthUtil.getSessionUser().getUserName());
        return mesProcessCellRDao.save(mesProcessCellDb);
    }

    @Override
    public int updateStatusProcessCell(long id, Integer status) {
        List<MesProcessCell> cellList=mesProcessCellRDao.findByProperty(new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED, MesExtConstWords.IS_VALID, MesExtConstWords.ID},
                new Object[]{AuthUtil.getOrganize().getOrganizeCode(), CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), MesExtEnumUtil.IS_VAILD.VAILD.getValue(), id});
        if(StringUtils.isEmpty(cellList)){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("该工序工位关系不存在")
                    .build();
        }
        return mesProcessCellRDao.updateByProperties(new String[]{MesExtConstWords.ID}, new Object[]{id},
                new String[]{MesConstWords.IS_VALID, MesExtConstWords.MODIFY_USER, MesExtConstWords.MODIFY_DATE_TIME},
                new Object[]{status, AuthUtil.getSessionUser().getUserName(), TimeTool.getNowTime(true)});
    }
}
