package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.jx;

import cn.estsh.i3plus.ext.mes.api.base.jx.IManageCodeService;
import cn.estsh.i3plus.ext.mes.api.base.jx.ISxWorkCellTaktCollectPlcCfgService;
import cn.estsh.i3plus.ext.mes.pojo.bean.SxWorkCellTaktCollectPlcCfg;
import cn.estsh.i3plus.ext.mes.pojo.repository.SxWorkCellTaktCollectPlcCfgRepository;
import cn.estsh.i3plus.ext.mes.pojo.sqlpack.MesExtHqlPack;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.tool.HqlPack;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Author: xinwang.yi
 * @CreateDate: 2023/10/09 11:41 上午
 * @Description:
 **/
@Service
public class SxWorkCellTaktCollectPlcCfgService implements ISxWorkCellTaktCollectPlcCfgService {

    @Autowired
    private SxWorkCellTaktCollectPlcCfgRepository workCellTaktCollectPlcCfgRepository;

    @Override
    public SxWorkCellTaktCollectPlcCfg insertSxWorkCellTaktCollectPlcCfg(SxWorkCellTaktCollectPlcCfg sxWorkCellTaktCollectPlcCfg) {

        //数据唯一性校验
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(sxWorkCellTaktCollectPlcCfg.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(sxWorkCellTaktCollectPlcCfg.getOrganizeCode(), MesExtConstWords.ORGANIZE_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(sxWorkCellTaktCollectPlcCfg.getWorkCenterCode(), MesExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(sxWorkCellTaktCollectPlcCfg.getWorkCellCode(), MesExtConstWords.WORK_CELL_CODE, packBean);
        SxWorkCellTaktCollectPlcCfg sxWorkCellTaktCollectPlcCfgIsExist = workCellTaktCollectPlcCfgRepository.getByProperty(packBean);

        if (sxWorkCellTaktCollectPlcCfgIsExist != null) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("产线代码【%s】工位代码【%s】在工位节拍PLC配置信息中已存在，状态为【%s】"
                            , sxWorkCellTaktCollectPlcCfgIsExist.getWorkCenterCode(), sxWorkCellTaktCollectPlcCfgIsExist.getWorkCellCode(), MesExtEnumUtil.IS_ENABLE.valueOfDescription(sxWorkCellTaktCollectPlcCfgIsExist.getStatus()))
                    .setErrorSolution("请更换新增参数")
                    .build();
        }

        ConvertBean.serviceModelInitialize(sxWorkCellTaktCollectPlcCfg, AuthUtil.getSessionUser().getUserName());
        return workCellTaktCollectPlcCfgRepository.insert(sxWorkCellTaktCollectPlcCfg);
    }

    @Override
    public void updateSxWorkCellTaktCollectPlcCfg(SxWorkCellTaktCollectPlcCfg sxWorkCellTaktCollectPlcCfg, String userName) {
        SxWorkCellTaktCollectPlcCfg sxWorkCellTaktCollectPlcCfgDb = workCellTaktCollectPlcCfgRepository.getById(sxWorkCellTaktCollectPlcCfg.getId());

        if (null == sxWorkCellTaktCollectPlcCfgDb)
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("ID[%s]记录不存在", sxWorkCellTaktCollectPlcCfg.getId())
                    .build();

        // 封装唯一校验查询条件
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(sxWorkCellTaktCollectPlcCfg.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(sxWorkCellTaktCollectPlcCfg.getWorkCenterCode(), MesExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(sxWorkCellTaktCollectPlcCfg.getWorkCellCode(), MesExtConstWords.WORK_CELL_CODE, packBean);
        SxWorkCellTaktCollectPlcCfg sxWorkCellTaktCollectPlcCfgIsExist = workCellTaktCollectPlcCfgRepository.getByProperty(packBean);

        if (null != sxWorkCellTaktCollectPlcCfgIsExist && sxWorkCellTaktCollectPlcCfgIsExist.getId().compareTo(sxWorkCellTaktCollectPlcCfgIsExist.getId()) != 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("产线代码【%s】工位代码【%s】在在工位节拍PLC配置信息启用或禁用数据中已存在"
                            , sxWorkCellTaktCollectPlcCfg.getWorkCenterCode(), sxWorkCellTaktCollectPlcCfg.getWorkCellCode())
                    .setErrorSolution("请更换新增参数")
                    .build();
        }

        sxWorkCellTaktCollectPlcCfgDb.setWorkCenterCode(sxWorkCellTaktCollectPlcCfg.getWorkCenterCode());
        sxWorkCellTaktCollectPlcCfgDb.setWorkCellCode(sxWorkCellTaktCollectPlcCfg.getWorkCellCode());
        sxWorkCellTaktCollectPlcCfgDb.setPlcCode(sxWorkCellTaktCollectPlcCfg.getPlcCode());
        sxWorkCellTaktCollectPlcCfgDb.setStatus(sxWorkCellTaktCollectPlcCfg.getStatus());
        sxWorkCellTaktCollectPlcCfgDb.setSystemSyncStatus(MesExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue());

        ConvertBean.serviceModelUpdate(sxWorkCellTaktCollectPlcCfg, userName);
        workCellTaktCollectPlcCfgRepository.save(sxWorkCellTaktCollectPlcCfg);
    }

    @Override
    public ListPager<SxWorkCellTaktCollectPlcCfg> querySxWorkCellTaktCollectPlcCfgByPager(SxWorkCellTaktCollectPlcCfg sxWorkCellTaktCollectPlcCfg, Pager pager) {
        if (sxWorkCellTaktCollectPlcCfg == null) {
            pager = PagerHelper.getPager(pager, workCellTaktCollectPlcCfgRepository.listCount());
            return new ListPager(workCellTaktCollectPlcCfgRepository.listPager(pager), pager);
        } else {
            DdlPackBean hqlPack = MesExtHqlPack.getSxWorkCellTaktCollectPlcCfg(sxWorkCellTaktCollectPlcCfg);
            hqlPack.setOrderByStr(sxWorkCellTaktCollectPlcCfg.orderBy());
            pager = PagerHelper.getPager(pager, workCellTaktCollectPlcCfgRepository.findByHqlWhereCount(hqlPack));
            List<SxWorkCellTaktCollectPlcCfg> byHqlWherePage = workCellTaktCollectPlcCfgRepository.findByHqlWherePage(hqlPack, pager);
            return new ListPager(byHqlWherePage, pager);
        }
    }

    @Override
    public int deleteWeaklySxWorkCellTaktCollectPlcCfgByIds(String[] ids, String userName) {

        StringBuffer wheres = new StringBuffer();
        HqlPack.getInPack(String.join(",", ids), MesExtConstWords.ID, wheres);

        int num = workCellTaktCollectPlcCfgRepository.updateByHqlWhere(
                wheres.toString(),
                new String[]{MesExtConstWords.SYSTEM_SYNC_STATUS, MesExtConstWords.IS_DELETED, MesExtConstWords.MODIFY_USER, MesExtConstWords.MODIFY_DATE_TIME},
                new Object[]{MesExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue(), CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue(), userName, TimeTool.getNowTime(true)}
        );
        return num;
    }

    @Override
    public int updateSxWorkCellTaktCollectPlcCfgStatus(String[] ids, int status, String userName) {
        StringBuffer wheres = new StringBuffer();
        HqlPack.getInPack(String.join(",", ids), MesExtConstWords.ID, wheres);

        int num = workCellTaktCollectPlcCfgRepository.updateByHqlWhere(
                wheres.toString(),
                new String[]{MesExtConstWords.SYSTEM_SYNC_STATUS, MesExtConstWords.STATUS, MesExtConstWords.MODIFY_USER, MesExtConstWords.MODIFY_DATE_TIME},
                new Object[]{MesExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue(), status, userName, TimeTool.getNowTime(true)}
        );
        return num;
    }

    @Override
    public int updateSxWorkCellTaktCollectPlcCfgValid(String[] ids, int status, String userName) {
        StringBuffer wheres = new StringBuffer();
        HqlPack.getInPack(String.join(",", ids), MesExtConstWords.ID, wheres);

        int num = workCellTaktCollectPlcCfgRepository.updateByHqlWhere(
                wheres.toString(),
                new String[]{MesExtConstWords.SYSTEM_SYNC_STATUS, MesExtConstWords.IS_VALID, MesExtConstWords.MODIFY_USER, MesExtConstWords.MODIFY_DATE_TIME},
                new Object[]{MesExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue(), status, userName, TimeTool.getNowTime(true)}
        );
        return num;
    }
}
