package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesConsistencyAffirmPart;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesPartExt;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesConsistencyAffirmPartRepository;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesPartExtRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.mes.apiservice.util.MesCommonUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : 一致性确认物料导入，支持编辑
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-04-23
 * @Modify:
 **/
@Service
public class ConsistencyAffirmPartExcelService implements IExcelImportExtService {

    @Autowired
    private MesConsistencyAffirmPartRepository consistencyAffirmPartRepository;

    @Autowired
    private MesPartExtRepository partExtRepository;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 一致性确认物料导入
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractMesConsistencyAffirmPart(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelMesConsistencyAffirmPart(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 一致性确认物料导入-读取表格
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractMesConsistencyAffirmPart(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesConsistencyAffirmPart集合
        List<MesConsistencyAffirmPart> consistencyAffirmPartList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的物料编码唯一校验，key：partNo，value：row
        Map<String, String> partNoMap = new HashMap<>();

        // 查询物料表
        Map<String, Object> partCodeEntityMap;
        try {
            partCodeEntityMap = MesCommonUtil.getCodeEntityMap(partExtRepository, MesExtConstWords.PART_NO, organizeCode, "物料表");
        } catch (Exception e) {
            partCodeEntityMap = null;
        }

        //查询一致性确认物料表
        List<MesConsistencyAffirmPart> consistencyAffirmParts = consistencyAffirmPartRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesConsistencyAffirmPart>> consistencyAffirmPartMap = CollectionUtils.isEmpty(consistencyAffirmParts) ? null :
                consistencyAffirmParts.stream().filter(o -> null != o).collect(Collectors.groupingBy(MesConsistencyAffirmPart::getPartNo));

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i ++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //物料编码
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String partNo = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                String partNameRdd = null;
                //生成规律
                String essences = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    essences = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                if (CollectionUtils.isEmpty(partCodeEntityMap) || !partCodeEntityMap.containsKey(partNo)) {
                    errorNum ++;
                    cellNum += "A;";
                    errorInfo += "第A列数据无效;";
                } else {
                    partNameRdd = ((MesPartExt)partCodeEntityMap.get(partNo)).getPartName();
                }

                boolean isExist = false;
                //校验物料编码不重复
                if (!partNoMap.containsKey(partNo)) {
                    partNoMap.put(partNo, String.valueOf(rowNum));
                    //校验 物料编码 是否已经存在数据库
                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(consistencyAffirmPartMap) && consistencyAffirmPartMap.containsKey(partNo)) {
                        isExist = true;
                    }
                } else {
                    errorNum ++;
                    cellNum += "A;";
                    errorInfo += "第A列数据跟第" + partNoMap.get(partNo) + "行的第A列数据重复;";
                }

                //校验生成规律
                if (StringUtils.isEmpty(essences)) {
                    errorNum ++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                } else {
                    try {
                        Double.valueOf(essences);
                        String[] essencesArr = essences.split(MesExtConstWords.DECIMAL_POINT);
                        if (null != essencesArr && essencesArr.length == 2 && essencesArr[1].length() > 2) {
                            errorNum ++;
                            cellNum += "B;";
                            errorInfo += "第B列数据无效,最多2位小数;";
                        }
                    } catch (NumberFormatException e) {
                        errorNum ++;
                        cellNum += "B;";
                        errorInfo += "第B列数据无效;";
                    }
                }

                //没有错误的时候，封装MesConsistencyAffirmPart
                if (errorNum == 0) {
                    MesConsistencyAffirmPart consistencyAffirmPart;
                    if(isExist){
                        consistencyAffirmPart = consistencyAffirmPartMap.get(partNo).get(0);
                        ConvertBean.serviceModelUpdate(consistencyAffirmPart, userName);
                    }else{
                        consistencyAffirmPart = new MesConsistencyAffirmPart();
                        ConvertBean.serviceModelInitialize(consistencyAffirmPart, userName);
                    }
                    consistencyAffirmPart.setPartNo(partNo);
                    consistencyAffirmPart.setPartNameRdd(partNameRdd);
                    consistencyAffirmPart.setEssences(Double.valueOf(essences));
                    consistencyAffirmPart.setOrganizeCode(organizeCode);

                    consistencyAffirmPartList.add(consistencyAffirmPart);

                    successRowNum ++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum ++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : consistencyAffirmPartList);
        return excelImportResultModel;
    }

    /**
     * 一致性确认物料扩展导入-数据入库
     * @param excelImportResultExtModel
     */
    private void insertExcelMesConsistencyAffirmPart(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesConsistencyAffirmPart> excelList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(excelList)) {
                for (MesConsistencyAffirmPart consistencyAffirmPart : excelList) {
                    if(StringUtils.isEmpty(consistencyAffirmPart.getId())){
                        consistencyAffirmPartRepository.insert(consistencyAffirmPart);
                    }else{
                        consistencyAffirmPartRepository.save(consistencyAffirmPart);
                    }
                }
            }
        }
    }
}
