package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesDataObject;
import cn.estsh.i3plus.pojo.mes.repository.MesDataObjectRepository;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:数据对象导入，支持编辑
 * @Reference:
 * @Author: wangjie
 * @CreateDate:2019\10\9
 * @Modify:
 **/
@Service
public class SxDataObjectExcelService implements IExcelImportExtService {

    @Autowired
    private MesDataObjectRepository dataObjectRepository;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 数据对象导入
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractDataObject(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelDataObject(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 数据对象导入-读取表格
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractDataObject(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesDataObject集合
        List<MesDataObject> dataObjectList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的对象代码唯一校验，key：objectCode，value：row
        Map<String,String> objectCodeMap = new HashMap<>();

        //查询数据对象表
        List<MesDataObject> dataObjects = dataObjectRepository.findByProperty(new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesDataObject>> dataObjectMap = dataObjects.stream().filter(o -> null != o && !StringUtils.isEmpty(o.getObjectCode()))
                .collect(Collectors.groupingBy(MesDataObject::getObjectCode));

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i ++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //对象代码
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String objectCode = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                //对象名称
                String objectName = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    objectName = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                boolean isExist = false;
                //校验对象代码
                if(!objectCodeMap.containsKey(objectCode)){
                    objectCodeMap.put(objectCode, String.valueOf(rowNum));
                    //校验数据库是否已经存在对象代码
                    if(!CollectionUtils.isEmpty(dataObjectMap) && dataObjectMap.containsKey(objectCode)){
                        isExist = true;
                    }
                }else{
                    errorNum ++;
                    cellNum += "A;";
                    errorInfo += "第A列数据与第" + objectCodeMap.get(objectCode) + "行的A列数据重复;";
                }

                //校验对象名称
                if(StringUtils.isEmpty(objectName)){
                    errorNum ++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                }

                //没有错误的时候，封装MesDataObject
                if (errorNum == 0) {
                    MesDataObject dataObject;
                    if(isExist){
                        dataObject = dataObjectMap.get(objectCode).get(0);
                        ConvertBean.serviceModelUpdate(dataObject, userName);
                    }else{
                        dataObject = new MesDataObject();
                        ConvertBean.serviceModelInitialize(dataObject, userName);
                    }
                    dataObject.setObjectCode(objectCode);
                    dataObject.setObjectName(objectName);
                    dataObject.setOrganizeCode(organizeCode);
                    dataObjectList.add(dataObject);
                    successRowNum ++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum ++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : dataObjectList);
        return excelImportResultModel;
    }

    /**
     * 数据对象导入-数据入库
     * @param excelImportResultExtModel
     */
    private void insertExcelDataObject(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesDataObject> excelList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(excelList)) {
                for (MesDataObject dataObject : excelList) {
                    if(StringUtils.isEmpty(dataObject.getId())){
                        dataObjectRepository.insert(dataObject);
                    }else{
                        dataObjectRepository.save(dataObject);
                    }
                }
            }
        }
    }
}
