package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesFourMConfirm;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesFourMConfirmRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : 4M确认导入，支持编辑
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-04-23
 * @Modify:
 **/
@Service
public class SxFourMConfirmExcelService implements IExcelImportExtService {

    @Autowired
    private MesFourMConfirmRepository fourMConfirmRepository;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 4M确认导入
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractMesFourMConfirm(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelMesFourMConfirm(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 4M确认导入-读取表格
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractMesFourMConfirm(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesFourMConfirm集合
        List<MesFourMConfirm> fourMConfirmList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的项目确认代码唯一校验，key：manageCode，value：row
        Map<String, String> confirmCodeMap = new HashMap<>();

        //查询4M确认表
        List<MesFourMConfirm> fourMConfirms = fourMConfirmRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesFourMConfirm>> fourMConfirmMap = CollectionUtils.isEmpty(fourMConfirms) ? null :
                fourMConfirms.stream().filter(o -> null != o).collect(Collectors.groupingBy(MesFourMConfirm::getConfirmCode));

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i ++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //项目确认代码
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String confirmCode = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                //管理类型
                String manageType = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    manageType = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //项目确认内容
                String confirmContent = null;
                try {
                    row.getCell(2).setCellType(CellType.STRING);
                    confirmContent = row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                boolean isExist = false;
                //校验项目确认代码不重复
                if (!confirmCodeMap.containsKey(confirmCode)) {
                    confirmCodeMap.put(confirmCode, String.valueOf(rowNum));
                    //校验 项目确认代码 是否已经存在数据库
                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(fourMConfirmMap) && fourMConfirmMap.containsKey(confirmCode)) {
                        isExist = true;
                    }
                } else {
                    errorNum ++;
                    cellNum += "A;";
                    errorInfo += "第A列数据跟第" + confirmCodeMap.get(confirmCode) + "行的第A列数据重复;";
                }

                //校验管理类型
                if (StringUtils.isEmpty(manageType)) {
                    errorNum ++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                } else {
                   if (StringUtils.isEmpty(MesExtEnumUtil.MANAGE_TYPE.descriptionOfValue(manageType))) {
                       errorNum ++;
                       cellNum += "B;";
                       errorInfo += "第B列数据无效;";
                   }
                }

                //校验项目确认内容
                if (StringUtils.isEmpty(confirmContent)) {
                    errorNum ++;
                    cellNum += "C;";
                    errorInfo += "第C列数据必填;";
                } else {
                    if (confirmContent.length() > 255) {
                        errorNum ++;
                        cellNum += "C;";
                        errorInfo += "第C列数据长度不能超过255个字符;";
                    }
                }

                //没有错误的时候，封装MesFourMManage
                if (errorNum == 0) {
                    MesFourMConfirm fourMConfirm;
                    if(isExist){
                        fourMConfirm = fourMConfirmMap.get(confirmCode).get(0);
                        ConvertBean.serviceModelUpdate(fourMConfirm, userName);
                    }else{
                        fourMConfirm = new MesFourMConfirm();
                        ConvertBean.serviceModelInitialize(fourMConfirm, userName);
                    }
                    fourMConfirm.setConfirmCode(confirmCode);
                    fourMConfirm.setConfirmContent(confirmContent);
                    fourMConfirm.setManageType(MesExtEnumUtil.MANAGE_TYPE.descriptionOfValue(manageType));
                    fourMConfirm.setOrganizeCode(organizeCode);

                    fourMConfirmList.add(fourMConfirm);

                    successRowNum ++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum ++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : fourMConfirmList);
        return excelImportResultModel;
    }

    /**
     * 4M确认扩展导入-数据入库
     * @param excelImportResultExtModel
     */
    private void insertExcelMesFourMConfirm(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesFourMConfirm> excelList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(excelList)) {
                for (MesFourMConfirm fourMConfirm : excelList) {
                    if(StringUtils.isEmpty(fourMConfirm.getId())){
                        fourMConfirmRepository.insert(fourMConfirm);
                    }else{
                        fourMConfirmRepository.save(fourMConfirm);
                    }
                }
            }
        }
    }
}
